@@ -153,23 +153,80 @@ def get_method_api_path(node_type):
             return None
 
 
-
-def get_params_for_get_method(response, node_type, url_var):
+def get_params_for_get_method(response, url_var, project_id):
     """
     Get params value to substitute variable in url
     """
     params = {}
     try:
+        var_list = [r"{bucket}", r"{policy}", r"{policy_name}"]
         for item in url_var:
-            if item == r'{bucket}':
+            if item in var_list:
                 params[item] = response['name']
-            else:
-                params[item] = response['projectId']
+            elif item == r"{project}" or item == r"{resource}":
+                try: 
+                    params[item] = response['projectId']
+                except:
+                    params[item] = project_id
+            elif item == r"{dataset}":
+                params[item] = response["datasetReference"]["datasetId"]
+            elif item == r"{account}":
+                try:
+                    params[item] = response['email']
+                except:
+                    account = response['name']
+                    account_val = account.split('/')[-3]
+                    params[item] = account_val
+
+            elif item == r"{key}" or item == r"{service_name}":
+                key_before_split = response['name']
+                key = key_before_split.split('/')[-1]
+                params[item] = key
+
         return params
     except Exception as ex:
         logger.error('Value not found: %s', ex)
         return params
 
+def get_request_url_get_method(get_method, item, project_id=None):
+    request_url, _ = get_method_api_path(get_method)
+    url_list = request_url.split('/')
+    url_var = []
+    for elements in url_list:
+        element = ''.join(re.findall(r'(?<={)[\d\D]*(?=})', elements))
+        if element:
+            url_var.append('{'+str(element)+'}')                    
+
+    params = get_params_for_get_method(item , url_var, project_id)
+    request_url = requested_get_method_url(request_url, params)
+    return request_url
+
+def get_request_url_list_method(get_method, list_method, item, project_id=None, credentials=None):
+    request_url = get_api_path(list_method)
+    url_list = request_url.split('/')
+    url_var = []
+    for elements in url_list:
+        element = ''.join(re.findall(r'(?<={)[\d\D]*(?=})', elements))
+        if element:
+            url_var.append('{'+str(element)+'}')                    
+
+    params = get_params_for_get_method(item , url_var, project_id)
+    request_url = requested_get_method_url(request_url, params)
+
+    access_token = credentials.get_access_token().access_token
+    header = {
+        "Authorization" : ("Bearer %s" % access_token)
+    }
+    list_data_response = requests.get(url=request_url, headers=header)
+    data = list_data_response.json()
+    resource_items =[]
+    resource_items = data['keys']
+    if resource_items:
+        for item in resource_items:
+            request_url = get_request_url_get_method(get_method, item, project_id)
+            return request_url
+
+
 def get_node(credentials, node, snapshot_source, snapshot):
     """
     Fetch node from google using connection. In this case using google client API's
@@ -208,6 +265,10 @@ def get_node(credentials, node, snapshot_source, snapshot):
 
         if get_method:
             node_type = node['get_method'] if node and 'get_method' in node else ""
+            if isinstance(node_type, list):
+                if len(node_type) > 1:
+                    node_type = ''.join(node_type[1])
+                node_type = ''.join(node_type)
             _, method = get_method_api_path(node_type)
             base_node_type_list = node_type.split("/")
             if len(base_node_type_list) > 1:
@@ -240,6 +301,10 @@ def get_node(credentials, node, snapshot_source, snapshot):
         else:
 
             node_type = node['type'] if node and 'type' in node else ""
+            if isinstance(node_type, list):
+                if len(node_type) > 1:
+                    get_method = ''.join(node_type[1])
+                node_type = ''.join(node_type)
             base_node_type_list = node_type.split("/")
             if len(base_node_type_list) > 1:
                 base_node_type = base_node_type_list[0]
@@ -322,7 +387,9 @@ def get_all_nodes(credentials, node, snapshot_source, snapshot, snapshot_data):
 
         fn_str_list = ""
         if node and 'type' in node and node['type']:
-            fn_str_list = get_field_value(node, 'type').split(".")
+            fn_str_list = get_field_value(node, 'type')
+            if isinstance(fn_str_list, list):
+                fn_str_list = str(fn_str_list[0]).split(".")
         
         response_param = ""
         if fn_str_list and len(fn_str_list) > 1:
@@ -341,20 +408,27 @@ def get_all_nodes(credentials, node, snapshot_source, snapshot, snapshot_data):
             data_filter = response_param.split("/")[-1]
 
             if "items" in data:
-                if isinstance(data['items'], dict):
-                    for name, scoped_dict in data['items'].items():
+                if isinstance(data["items"], dict):
+                    for name, scoped_dict in data["items"].items():
                         if response_param in scoped_dict:
                             db_record['items'] = db_record['items'] + scoped_dict[check_node_type]
-
-                if not db_record['items']:
-                    db_record['items'] = data['items']
+                        elif data_filter in scoped_dict:
+                            db_record['items'] = db_record['items'] + scoped_dict[data_filter]
+                elif "items" in data and not db_record['items']:
+                    db_record['items'] = data["items"]
             elif data_filter in data:
                 db_record['items'] = data[data_filter]
+                
+            elif "items" not in data and data_filter not in data:
+                list_var = []
+                list_var.append(data)
+                response_data = {"items" : list_var}
+                db_record['items'] = response_data["items"]
             
             # snapshot_data["project-id"] = project_id
             # snapshot_data["request_url"] = request_url
             
-            set_snapshot_data(node, db_record['items'], snapshot_data)
+            set_snapshot_data(node, db_record['items'], snapshot_data, project_id, credentials)
 
             checksum = get_checksum(data)
             if checksum:
@@ -362,13 +436,19 @@ def get_all_nodes(credentials, node, snapshot_source, snapshot, snapshot_data):
 
     return db_record
 
-def set_snapshot_data(node, items, snapshot_data):
+def set_snapshot_data(node, items, snapshot_data, project_id=None, credentials=None):
     if node['masterSnapshotId'] not in snapshot_data or not isinstance(snapshot_data[node['masterSnapshotId']], list):
         snapshot_data[node['masterSnapshotId']] =  []
 
     # create the node type for sub resources
     node_type = get_field_value(node, "type")
     get_method = get_field_value(node, "get_method")
+    list_method = None
+    if isinstance(get_method, list):
+        if len(get_method) > 1:
+            list_method = ''.join(get_method[0])
+            get_method = ''.join(get_method[1])
+        get_method = ''.join(get_method)
     node_type_list = node_type.split(".")
     resource_node_type = node_type
     if len(node_type_list) > 1:
@@ -414,18 +494,11 @@ def set_snapshot_data(node, items, snapshot_data):
     for item in resource_items:
         count += 1
 
-        if get_method:
-            request_url, _ = get_method_api_path(get_method)
-            url_list = request_url.split('/')
-            url_var = []
-            for elements in url_list:
-                element = ''.join(re.findall(r'(?<={)[\d\D]*(?=})', elements))
-                if element:
-                    url_var.append('{'+str(element)+'}')                    
-
-            params = get_params_for_get_method(item ,get_method, url_var)
-            request_url = requested_get_method_url(request_url, params)
-             
+        if get_method and not list_method:
+            request_url = get_request_url_get_method(get_method, item, project_id)
+        
+        elif list_method:
+            request_url = get_request_url_list_method(get_method, list_method, item, project_id, credentials)
 
         else:
             if "selfLink" in item.keys():
